//  
//  Copyright (c) 1994 by Mike Romberg (  romberg@md.fsl.noaa.gov )
//
//  This file may be distributed under terms of the GPL
//

#include "fieldmeter.h"
#include "xosview.h"
#include <fstream.h>

FieldMeter::FieldMeter( XOSView *parent, int x, int y, int width, int height, 
		        int numfields, const char *title, const char *legend,
		        int dolegends){
  dolegends_ = dolegends;
  title_ = legend_ = NULL;
  FieldMeter::title( title );
  FieldMeter::legend( legend );
  parent_ = parent;
  resize( x, y, width, height );
  numfields_ = numfields;
  fields_ = new float[numfields_];
  colors_ = new XColor[numfields_];
  lastvals_ = new int[numfields_];
  lastx_ = new int[numfields_];

  for ( int i = 0 ; i < numfields_ ; i++ ){
    fields_[i] = lastvals_[i] = lastx_[i] = 0;
    setfieldcolor( i, "blue" );
  }
}

FieldMeter::~FieldMeter( void ){
  delete[] fields_;
  delete[] colors_;
  delete[] title_;
  delete[] legend_;
  delete[] lastvals_;
  delete[] lastx_;
}

void FieldMeter::title( const char *title ){
  delete[] title_;
  title_ = new char[strlen( title ) + 1];
  strcpy( title_, title );
}

void FieldMeter::reset( void ){
  for ( int i = 0 ; i < numfields_ ; i++ )
    lastvals_[i] = lastx_[i] = -1;
}

void FieldMeter::legend( const char *legend ){
  delete[] legend_;
  legend_ = new char[strlen( legend ) + 1];
  strcpy( legend_, legend );
}

void FieldMeter::resize( int x, int y, int width, int height ){
  x_ = x;
  y_ = y;
  width_ = width;
  height_ = height;
}


void FieldMeter::setfieldcolor( int field, char *color ){
  XAllocNamedColor( parent_->display(),
		    DefaultColormap( parent_->display(), parent_->screen() ),
		    color, &colors_[field], &colors_[field] );  
}

void FieldMeter::draw( void ){
  parent_->setforegroundcolor( "white" );
  XDrawRectangle( parent_->display(), parent_->window(), parent_->gc(),
		  x_ - 1, y_ - 1, width_ + 1, height_ + 1);
  if ( dolegends_ ){
    parent_->setforegroundcolor( "yellow" );
    parent_->drawstr( x_ - 25, y_ + height_, title_ );
    drawlegend();
  }
}

void FieldMeter::drawlegend( void ){
  char *tmp1, *tmp2, buff[100];
  int n, x = x_;
  struct XFontStruct *xfont = XQueryFont(parent_->display(), parent_->font());
  
  tmp1 = tmp2 = legend_;
  for ( int i = 0 ; i < numfields_ ; i++ ){
    n = 0;
    while ( (*tmp2 != '/') && (*tmp2 != '\0') ){
      tmp2++;
      n++;
    }
    tmp2++;
    strncpy( buff, tmp1, n );
    buff[n] = '\0';
    XSetForeground( parent_->display(), parent_->gc(), colors_[i].pixel );
    parent_->drawstr( x, y_ - 5, buff );
    x += XTextWidth( xfont, buff, n );
    parent_->setforegroundcolor( "white" );
    if ( i != numfields_ - 1 )
      parent_->drawstr( x, y_ - 5, "/" );    
    x += XTextWidth( xfont, "/", 1 );
    tmp1 = tmp2;
  }
}

void FieldMeter::drawfields( void ){
  int twidth, x = x_;

  for ( int i = 0 ; i < numfields_ ; i++ ){
    twidth = (int)((fields_[i] * width_) / total_);
    if ( (i == numfields_ - 1) && ((x + twidth) != (x_ + width_)) ){
      twidth = width_ + x_ - x;
    }

    if ( (twidth != lastvals_[i]) || (x != lastx_[i]) ){
      XSetForeground( parent_->display(), parent_->gc(), colors_[i].pixel );
      XFillRectangle( parent_->display(), parent_->window(), parent_->gc(),
		     x, y_, twidth, height_ );
      lastvals_[i] = twidth;
      lastx_[i] = x;
    }
    x += twidth;
  }

  parent_->flush();
}

void FieldMeter::checkevent( void ){
  drawfields();
}
