//  
//  Copyright (c) 1994 by Mike Romberg (  romberg@md.fsl.noaa.gov )
//
//  This file may be distributed under terms of the GPL
//

#include "xosview.h"
#include "cpumeter.h"
#include "memmeter.h"
#include "swapmeter.h"
#include "netinmeter.h"
#include "netoutmeter.h"
#include <iostream.h>
#include <unistd.h>
#include <string.h>


#define NAME "xosview@"


XOSView::XOSView( int argc, char *argv[] )
:XWin( argc, argv, findx( argc, argv ), findy( argc, argv ), winname(), 
       "", "", "5x8" ){
  xoff_ = 5;
  yoff_ = 0;
  parseargs( argc, argv );
  if ( netmax_ > 0 )
    nummeters_ = 5;
  else
    nummeters_ = 3;
  cpu_ = new CPUMeter( this, xoff_, 15, width_ - 10, 10 );
  mem_ = new MemMeter( this, xoff_, 40, width_ - 10, 10 );
  swap_ = new SwapMeter( this, xoff_, 65, width_ - 10, 10 );
  if ( netmax_ > 0 ){
    netin_ = new NetInMeter( this, xoff_, 90, width_ - 10, 10, netmax_ );
    netout_ = new NetOutMeter( this, xoff_, 115, width_ - 10, 10, netmax_ );
  }
  cpu_->dolegends( legend_ );
  mem_->dolegends( legend_ );
  swap_->dolegends( legend_ );
  if ( netmax_ > 0 ){
    netin_->dolegends( legend_ );
    netout_->dolegends( legend_ );
  }
  done_ = 0;
  resize();
}

int XOSView::findx( int argc, char *argv[] ){
  parseargs( argc, argv );
  if ( legend_ )
    return 122;
  return 80;
}

int XOSView::findy( int argc, char *argv[] ){
  parseargs( argc, argv );
  if ( legend_ ){
    if ( netmax_ > 0 )
      return 115;
    return 80;
  }
  if ( netmax_ > 0 )
    return 85;
  return 45;
}

void XOSView::parseargs( int argc, char *argv[] ){
  int i = 1;
  netmax_ = legend_ = 0;
  while ( i <= argc - 1 ){
    if ( !strcmp( argv[i], "-l" ) ){
      legend_ = 1;
      xoff_ = 25;
      yoff_ = 10;
    }
    else if ( !strcmp( argv[i], "-net" ) ){
      i++;
      if ( i >= argc ){
	cerr <<"XOSView::parseargs() : no parameter for option -net" <<endl;
	exit( 1 );
      }
      netmax_ = atof( argv[i] );
    }
    i++;
  }
}

const char *XOSView::winname( void ){
  static char name[100];
  char host[100];
  strcpy( name, NAME );
  gethostname( host, 99 );
  strcat( name, host );
  return name;
}


void  XOSView::resize( void ){
  int newwidth = width_ - 10;
  int newheight = (height_ - (10 + 5 * (nummeters_ - 1) + 
			      nummeters_ * yoff_)) / nummeters_;
  cpu_->resize( xoff_, 5 + yoff_, newwidth, newheight );
  mem_->resize( xoff_, 10 + (2 * yoff_) + newheight, newwidth, newheight );
  swap_->resize( xoff_, 15 +(3 * yoff_) + 2 * newheight, newwidth, newheight );
  if ( netmax_ > 0 ){
    netin_->resize(xoff_, 20 +(4 * yoff_) + 3 * newheight, 
		   newwidth, newheight);
    netout_->resize(xoff_, 25 +(5 * yoff_) + 4 * newheight, 
		    newwidth, newheight);
  }
  XWin::resize();
}

XOSView::~XOSView( void ){
  delete cpu_;
  delete mem_;
  delete swap_;
  if ( netmax_ > 0 ){
    delete netin_;
    delete netout_;
  }
}

void XOSView::draw( void ){
  cpu_->draw();
  mem_->draw();
  swap_->draw();
  if ( netmax_ > 0 ){
    netin_->draw();
    netout_->draw();
  }
}

void XOSView::keyrelease( char *ch ){
  if ( (*ch == 'q') || (*ch == 'Q') )
    done_ = 1;
}

void XOSView::run( void ){
  int counter = 0;

  while( !done_ ){
    checkevent();
    cpu_->checkevent();
    if ( netmax_ > 0 ){
      NetMeter::checkevent();
      netin_->checkevent();
      netout_->checkevent();
    }
    if ( !counter ) {
      mem_->checkevent();
      swap_->checkevent();
    }

    flush();
    usleep( 200000 );
    counter = (counter + 1) % 5;    
  }
}

void XOSView::visnotify( void ){
  cpu_->reset();
  mem_->reset();
  swap_->reset();
  if ( netmax_ > 0 ){
    netin_->reset();
    netout_->reset();
  }

  draw();
}
