//  
//  Copyright (c) 1994 by Mike Romberg (  romberg@md.fsl.noaa.gov )
//
//  This file may be distributed under terms of the GPL
//

#ifndef _xwin_h_
#define _xwin_h_

#include <stdlib.h>
#include <string.h>
extern "C"
{
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>	/* For erasing cursor - not important	*/
}

class XWin {
public:
  XWin( int width, int height, const char *title, const char *foreground = "", 
        const char *background = "", const char *font = "9x15" );
  XWin( int argc, char *argv[], int width, int height, const char *title, 
        const char *foreground = "", const char *background = "", 
        const char *font = "9x15" );
  virtual ~XWin();
  virtual void draw( void )=0;
  virtual void visnotify( void ) { draw(); }
  virtual void mapnotify( void ) { draw(); }
  virtual void clearwin();
  virtual int  setforegroundcolor( const char *name );
  virtual int  setbackgroundcolor( const char *name );
  void drawstr( int x, int y, const char *str ) 
    { XDrawImageString( disp_, window_, gc_, x, y, str, strlen( str ) ); }
  virtual void resize();
  void resize( int width, int height ){ 
    width_ = width;  height_ = height; 
    XResizeWindow( disp_, window_, width_, height_ ); 
  }
  virtual void keyrelease(char *ch);
  virtual void buttonrelease();
  virtual void buttonpress();
  virtual void checkevent();
  void map( void ) { XMapWindow( disp_, window_ ); }
  void unmap( void ) { XUnmapWindow( disp_, window_ ); }
  Display *display( void ) { return disp_; }
  Window window( void ) { return window_; }
  int height( void ) { return height_; }
  int width( void ) { return width_; }
  const char *title( void ) { return title_; }
  void title( const char *title );
  const char *iconname( void ) { return iconname_; }
  void iconname( const char *name );
  int screen( void ) { return screen_; }
  int depth( void ) { return depth_; }
  int whitepixel( void ) { return whitepixel_; }
  int blackpixel( void ) { return blackpixel_; }
  GC gc( void ) { return gc_; }
  Font font( void ) { return font_; }
  void flush( void ) { XFlush( disp_ ); }
  int positionit( void ) { return positionit_; }
protected:
  Display *disp_;
  Window window_;
  Cursor cursor_;
  int height_, width_, screen_, depth_, whitepixel_, blackpixel_;
  char *title_, *iconname_;
  GC gc_;
  Font font_;
  XSizeHints hint_;
  int positionit_;

  void init( int width, int height, const char *title, 
	     const char *foreground, const char *background,
	     const char *font );
private:
};

#endif
